/*
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Wenjie Xiang <xiangwenjie@kylinos.cn>
 *
 */

#include "multi-select-combobox.h"
#include <QLineEdit>
#include <QEvent>
#include <QMouseEvent>
#include <QDebug>

using namespace Peony;
MultiSelectComboBox::MultiSelectComboBox(QWidget *parent)
    : QComboBox(parent)
{
    m_lineEdit = new QLineEdit(this);
    m_lineEdit->setReadOnly(true);
    this->setLineEdit(m_lineEdit);
    this->lineEdit()->disconnect();
    m_lineEdit->installEventFilter(this);

    m_listView = new QListView(this);
    this->setView(m_listView);
    this->setModel(&m_model);

    connect(this, QOverload<int>::of(&MultiSelectComboBox::activated), this, &MultiSelectComboBox::slotActivated);
}

MultiSelectComboBox::~MultiSelectComboBox()
{

}

void MultiSelectComboBox::addItem(const QString &text, bool checked, const QVariant &userData)
{
    QStandardItem *item = new QStandardItem(text);
    item->setCheckable(true);
    item->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
    item->setData(userData, Qt::UserRole + 1);
    m_model.appendRow(item);

    this->updateText();
}

void MultiSelectComboBox::addItems(const QStringList &texts)
{
    for (auto text : texts) {
        this->addItem(text);
    }
}

void MultiSelectComboBox::removeItem(int index)
{
    m_model.removeRow(index);
    this->updateText();
}

void MultiSelectComboBox::clear()
{
    m_model.clear();
    this->updateText();
}

QStringList MultiSelectComboBox::getSelectItemsText()
{
    QStringList texts;
    QString str = m_lineEdit->text();
    if (!str.isEmpty()) {
        texts = m_lineEdit->text().split(",");
        return texts;
    }
    return texts;
}

QList<int> MultiSelectComboBox::getSelectItemsIndex()
{
    QList<int> indexs;
    for (int i = 0; i < m_model.rowCount(); i++) {
        QStandardItem *item = m_model.item(i);
        if (item->checkState() == Qt::Checked) {
            indexs.append(i);
        }
    }
    return indexs;
}

QString MultiSelectComboBox::getItemText(int index)
{
    if (index < 0 || index >= m_model.rowCount()) {
        return QString("");
    }

    return m_model.item(index)->text();
}

void MultiSelectComboBox::setDefaultItem(int index)
{
    if (index < 0 || index >= m_model.rowCount()) {
        return;
    }

    QStandardItem *item = m_model.item(index);
    item->setCheckState(Qt::Checked);
    this->updateText();
}

void MultiSelectComboBox::setEnableItem(int index, bool enabled)
{
    if (index < 0 || index >= m_model.rowCount()) {
        return;
    }

    QStandardItem *item = m_model.item(index);
    item->setEnabled(enabled);
    this->updateText();
}

QString MultiSelectComboBox::getShowText()
{
    QStringList selectedTexts;
    for (int i = 0; i < m_model.rowCount(); i++) {
        QStandardItem *item = m_model.item(i);
        if (item->checkState() == Qt::Checked) {
            selectedTexts.append(item->text());
        }
    }

    return selectedTexts.join(",  ");
}

void MultiSelectComboBox::setCheckedItem(int index, bool checked)
{
    if (index < 0 || index >= m_model.rowCount()) {
        return;
    }

    QStandardItem *item = m_model.item(index);
    item->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
    this->updateText();
}

void MultiSelectComboBox::uncheckAllItem()
{
    for (int i = 0; i < m_model.rowCount(); i++) {
        this->setCheckedItem(i);
    }
}

void MultiSelectComboBox::showPopup()
{
    Q_EMIT showingPopup();
    QComboBox::showPopup();
}

void MultiSelectComboBox::hidePopup()
{
    QRect rectView = this->view()->geometry();
    rectView.moveTopLeft(this->view()->mapToGlobal(QPoint(0, 0)));

    QPoint globalPos = QCursor::pos();

    if (!rectView.contains(globalPos)) {
        Q_EMIT hidingPopup();
        QComboBox::hidePopup();
    }
}

void MultiSelectComboBox::mousePressEvent(QMouseEvent *event)
{
    QComboBox::mousePressEvent(event);
    event->accept();
}

void MultiSelectComboBox::mouseReleaseEvent(QMouseEvent *event)
{
    QComboBox::mouseReleaseEvent(event);
    event->accept();
}

void Peony::MultiSelectComboBox::mouseMoveEvent(QMouseEvent *event)
{
    QComboBox::mouseMoveEvent(event);
    event->accept();
}

bool MultiSelectComboBox::eventFilter(QObject *watched, QEvent *event)
{
    if (watched == m_lineEdit && event->type() == QEvent::MouseButtonRelease) {
        showPopup();
        return true;
    }
    return false;
}

void MultiSelectComboBox::resizeEvent(QResizeEvent *e)
{
    QComboBox::resizeEvent(e);
    updateText();
}

void MultiSelectComboBox::updateText()
{
    QStringList selectedTexts;
    for (int i = 0; i < m_model.rowCount(); i++) {
        QStandardItem *item = m_model.item(i);
        if (item->checkState() == Qt::Checked) {
            selectedTexts.append(item->text());
        }
    }

    QString displayText = selectedTexts.join(",  ");
    int width = this->width() - 35;
    if (m_lineEdit->fontMetrics().width(displayText) > width) {
        displayText = m_lineEdit->fontMetrics().elidedText(displayText, Qt::ElideRight, width);
    }

    m_lineEdit->setText(displayText);
    m_lineEdit->setToolTip(selectedTexts.join("\n"));
}

void MultiSelectComboBox::slotActivated(int index)
{
    QStandardItem *item = m_model.item(index);
    if (nullptr == item) {
        return;
    }

    Qt::CheckState state = (item->checkState() == Qt::Checked) ? Qt::Unchecked : Qt::Checked;
    item->setCheckState(state);

    this->updateText();
    Q_EMIT currentItemStateChanged();
}
